<?php

namespace Edulab\College\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Edulab\College\Forms\Teacher as TeacherForm;
use Edulab\College\Http\Requests\TeacherResourceRequest;
use Edulab\College\Http\Resources\TeacherResource;
use Edulab\College\Http\Resources\TeachersCollection;
use Edulab\College\Models\Teacher;
use Edulab\College\Actions\TeacherAction;
use Edulab\College\Actions\TeacherActions;


/**
 * Resource controller class for teacher.
 */
class TeacherResourceController extends BaseController
{

    /**
     * Initialize teacher resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = TeacherForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('edulab.college.modules'), 'college', guard_url('college'));
            return $next($request);
        });
    }

    /**
     * Display a list of teacher.
     *
     * @return Response
     */
    public function index(TeacherResourceRequest $request)
    {
        $request = $request->all();
        $page = TeacherActions::run('paginate', $request);

        $data = new TeachersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('college::teacher.names'))
            ->view('college::teacher.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display teacher.
     *
     * @param Request $request
     * @param Model   $teacher
     *
     * @return Response
     */
    public function show(TeacherResourceRequest $request, Teacher $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TeacherResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('college::teacher.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('college::teacher.show')
            ->output();
    }

    /**
     * Show the form for creating a new teacher.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TeacherResourceRequest $request, Teacher $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TeacherResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('college::teacher.name'))
            ->view('college::teacher.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new teacher.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TeacherResourceRequest $request, Teacher $model)
    {
        try {
            $request = $request->all();
            $model = TeacherAction::run('store', $model, $request);
            $data = new TeacherResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('college::teacher.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('college/teacher/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/college/teacher'))
                ->redirect();
        }

    }

    /**
     * Show teacher for editing.
     *
     * @param Request $request
     * @param Model   $teacher
     *
     * @return Response
     */
    public function edit(TeacherResourceRequest $request, Teacher $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new TeacherResource($model);
        // return view('college::teacher.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('college::teacher.name'))
            ->view('college::teacher.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the teacher.
     *
     * @param Request $request
     * @param Model   $teacher
     *
     * @return Response
     */
    public function update(TeacherResourceRequest $request, Teacher $model)
    {
        try {
            $request = $request->all();
            $model = TeacherAction::run('update', $model, $request);
            $data = new TeacherResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('college::teacher.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('college/teacher/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('college/teacher/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the teacher.
     *
     * @param Model   $teacher
     *
     * @return Response
     */
    public function destroy(TeacherResourceRequest $request, Teacher $model)
    {
        try {

            $request = $request->all();
            $model = TeacherAction::run('destroy', $model, $request);
            $data = new TeacherResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('college::teacher.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('college/teacher/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('college/teacher/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
